#ifndef MANALINK_H
#define MANALINK_H

#include "defs.h"

#include <list>

/*
* class loseGameRecorder
*
* Use this class to establish an "artifical phase" around actions that should
* happen simulatanously where each action might trigger lose_the_game().
* Example: both players draw cards (Prosperity), both would run out of cards
* but the player drawing first loses the game although it should end in a draw.
*
*  Methods:
*    instance: get an instance of loseGameRecorder.
*    startRecording: initiates recording of lose_the_game calls inside
*      lose_the_game. The game will not end due to lose_the_game calls.
*    stopRecording: stops recording of lose_the_game calls. If calls happened
*      during recording the calls are evaluated and lose_the_game is called.
*	   If both players or ANYBODY would lose the game the game ends in a draw.
*	   Every call of startRecording must have a corresponding call of
*	   stopRecording. lose_the_game is called after the last stopRecording.
*    appendCall: called by lose_the_game to add the current call.
*
*  Example:
*   void doSomethingSimultanously(player_t player, ...) {
*	   loseGameRecorder::instance().startRecording();
*      doSomething(player);
*      doSomethingElse(1-player);
*      loseGameRecorder::instance().stopRecording();
*	   return;
*   }
*
* Remarks:
*  The implemenation is thread-safe, lazily evaluated and does not require
*  garbage collection. All memory is allocated on the stack.
*  Exwecption handling is done inside the class. An error message is shown
*  and the game ends in a draw.
*/

class loseGameRecorder
{
 public:
    static loseGameRecorder& instance()
    {
       static loseGameRecorder _instance;
	   return _instance;
    }

    ~loseGameRecorder() {}
    
    void startRecording() { myMutex++; return; };
    void stopRecording();
    int isRecording() { return (myMutex != 0); };
    
    void appendCall(player_t call);

 private:
    loseGameRecorder():myMutex(0) {} // to prevent outside creation of instances
    loseGameRecorder( const loseGameRecorder& ); // to prevent outside creation of instances through copy constructor
    loseGameRecorder & operator = (const loseGameRecorder &); // to prevent outside creation of instances through copying
    
    player_t determineLooser();

    int myMutex;
 	std::list<player_t> myCalls;
};

void lose_the_game(player_t player);
void real_lose_the_game(player_t player);

int both_players_draw_naive(int amount);
int both_players_draw_better(int amount);
int draw_cards(player_t player, int amount);

void laboratoryManiac(player_t player);

#endif
